//+------------------------------------------------+
//| Boll M5 Last Update 1-09-2007 12:00am
//+------------------------------------------------+

#property copyright "Skyview Innovations, LLC"
#property link      " "
#define     NL      "\n" 
#include <stdlib.mqh>

// user input
extern double Lots         =  3;      // how many lots to trade at a time
extern double Increment    =  3;
extern double PipTarget    =  5;
extern double Step         =  10;
extern bool   EndTrading   =  false;  // Closes EA after all trades close
extern int    SL           =  45;
extern int    EndHour      =  6;
extern int    StartHour    =  17;

int    BPeriod             =  1;       // Bollinger period
int    Deviation           =  1;       // Bollinger deviation

// non-external flag settings
int    Slippage=2;                     // how many pips of slippage can you tolorate
bool   OneOrderOnly=false ;            // one order at a time or not

// naming and numbering
int    MagicNumber  = 46566735;     // allows multiple experts to trade on same account
string TradeComment = "Boll M5";   // comment so multiple EAs can be seen in Account History

// Bar handling
datetime bartime=0;                    // used to determine when a bar has moved

// Trade control
bool   TradeAllowed=true;              // used to manage trades


//+-------------+
//| Custom init |
//|-------------+
// Called ONCE when EA is added to chart or recompiled

int init()
  {
  }

//+----------------+
//| Custom DE-init |
//+----------------+
// Called ONCE when EA is removed from chart

int deinit()
  {
  
  }
    
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+
void CloseAllThisSymbolBuy()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)  
      {
           //Close opened long positions
           if ( OrderType() == OP_BUY)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 5, Red );
      }
  }
  return; 
  }
// && OrderProfit()>0    
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+
void CloseAllThisSymbolSell()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
      {
           //Close opened short positions
           if ( OrderType() == OP_SELL)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 5, Red );
      }
  }
  return; 
  }
   
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double   i, TotalProfit, BuyProfit, SellProfit, BuyPips, SellPips;
   double   TotalLotsBuy, TotalLotsSell;
   int      BuyLots, SellLots, OrdersBUY, OrdersSELL;
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
       if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
        {
         if (OrderType() == OP_BUY) { BuyProfit += OrderProfit() + OrderCommission() + OrderSwap();
                                      BuyLots+=OrderLots();
                                      OrdersBUY++; }
         if (OrderType() == OP_SELL){ SellProfit += OrderProfit() + OrderCommission() + OrderSwap();
                                      SellLots+=OrderLots();
                                      OrdersSELL++; }
        }
     }
    
  //---- close conditions 
    BuyPips   = PipTarget * BuyLots + OrdersSELL*10;   
    SellPips  = PipTarget * SellLots + OrdersBUY*10;
    
    if(BuyProfit  > BuyPips) CloseAllThisSymbolBuy(); 
    if(SellProfit > SellPips) CloseAllThisSymbolSell();
    
    return;
  }
  
//+------------------------------------------------------------------+
//| Check for BUY order conditions                                   |
//+------------------------------------------------------------------+
void CheckForBuy()
  {
   double   BuyOrders, LowestBuy=1000;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   
      // bar counting
   if(bartime!=Time[0]) 
     {
      bartime=Time[0];
      TradeAllowed=true;
     }
     
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderType() == OP_BUY && OrderMagicNumber()==MagicNumber)
        {
            if (OrderOpenPrice()<LowestBuy) LowestBuy=OrderOpenPrice();
            BuyOrders++;
        }
     }
      
  //ENTRY LONG (buy, Ask) 
   if(TradeAllowed && Ask < LowestBuy)
     {
         ticket = OrderSend(Symbol(),OP_BUY,Lots+(BuyOrders*Increment),Ask,Slippage,Ask-SL*Point,0,TradeComment,MagicNumber,Red);
         gle=GetLastError();
         if(gle==0)
           {
            TradeAllowed=false;
           }
     }//SELLme
   }
    
//+------------------------------------------------------------------+
//| Check for SELL order conditions                                   |
//+------------------------------------------------------------------+
void CheckForSell ()
  {
   double   SellOrders, HighestSell;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;   
      // bar counting
   if(bartime!=Time[0]) 
     {
      bartime=Time[0];
      TradeAllowed=true;
     }
     
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderType() == OP_SELL && OrderMagicNumber()==MagicNumber)
        {
            if (OrderOpenPrice()>HighestSell) HighestSell=OrderOpenPrice();
            SellOrders++;
        }
     } 
    
    //ENTRY SHORT (sell, Bid)
   if(TradeAllowed && Bid > HighestSell)
     {
         ticket = OrderSend(Symbol(),OP_SELL,Lots+(SellOrders*Increment),Bid,Slippage,Bid+SL*Point,0,TradeComment,MagicNumber,Red);
         gle=GetLastError();
         if(gle==0)
           {
            TradeAllowed=false;
           }
     }//SELLme
   }
      
//+-----------+
//| Main      |
//+-----------+
// Called EACH TICK and each Bar[]
int start()
  {
  
   CheckForClose();
   
   double   OrdersSELL, OrdersBUY, BuyTarget, SellTarget, CounterEntry;
   double   BuyLots, SellLots, BuyProfit, SellProfit, ProfitTarget, LargestMargin;
   int      CurrentTime = TimeHour(CurTime());
   bool     SELLme=false;
   bool     BUYme=false;
   double   Stoch1 = iStochastic(NULL,60,100,22,1,MODE_LWMA,0,MODE_SIGNAL,0);
   double   Stoch2 = iStochastic(NULL,60,100,22,1,MODE_LWMA,0,MODE_SIGNAL,1);
   double   Spread = Ask-Bid;
   
   for(int i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
       if (OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
           if(OrderType()==OP_BUY)  OrdersBUY++;
           if(OrderType()==OP_SELL) OrdersSELL++;
           if(OrderType()==OP_BUY)  BuyLots += OrderLots();
           if(OrderType()==OP_SELL) SellLots += OrderLots();
           if (OrderType() == OP_BUY)  BuyProfit += OrderProfit() + OrderCommission() + OrderSwap();
           if (OrderType() == OP_SELL) SellProfit += OrderProfit() + OrderCommission() + OrderSwap();
        }
     }

   if (Bid>Open[0]+(Step*Point) && Spread<=4) SELLme=true; 
   if (EndTrading == true && OrdersSELL == 0) SELLme = false;
   if (CurrentTime >= EndHour && CurrentTime < StartHour && OrdersSELL == 0) SELLme =false; 

   if (Ask<Open[0]-(Step*Point) && Spread<=4) BUYme=true;  
   if (EndTrading == true && OrdersBUY == 0) BUYme = false;
   if (CurrentTime >= EndHour && CurrentTime < StartHour && OrdersBUY == 0) BUYme = false;

   if (SELLme == true) CheckForSell();
   if (BUYme == true) CheckForBuy();
   
   Comment("                    Boll M5 Last Update 1-9-2007 12:00am", NL,
           "                            Buys    ", OrdersBUY, NL,
           "                            BuyLots        ", BuyLots, NL,
           "                            Sells    ", OrdersSELL, NL,
           "                            SellLots        ", SellLots, NL,
           "                            Balance ", AccountBalance(), NL,
           "                            Equity        ", AccountEquity(), NL,
           "                            Margin              ", AccountMargin(), NL,
           "                            MarginPercent        ", MathRound((AccountEquity()/AccountMargin())*100), NL,
           "                            Current Time is  ",TimeHour(CurTime()),":",TimeMinute(CurTime()),".",TimeSeconds(CurTime()));   
 } // start()




