//+------------------------------------------------------------------+
//|                                                  DropKickv16.mq4 |
//|                                       Copyright  2006, kmrunner |
//|                                                    July 31, 2006 |
//+------------------------------------------------------------------+
#define MAGICMA  20060731
#include <stdlib.mqh>
#define     NL    "\n"

extern double PercentRisk     = 5;      // CAUTION: PERCENT RISK FOR EACH CURRENCY
extern double LotDivider      = 80;     // divides your max allowed lots by this number
extern double LargestLotAllowed = 0.5;   // largest lot allowed to open
extern bool WantBuys          = true;
extern bool WantSells         = false;
extern bool WantTakeProfit    = false;   // Sets orders with a TP level
extern int TakeProfit         = 30;     // Take profit level for TakeProfit option
extern int vol                = 4;      // sets trades for this many ticks of current bar
extern int CloseAllAbove      = 0.1;    // closes all trades above this amount if close conditions are met
extern int RSIperiod          = 3;      // RSI setting on "shortterm" time frame
extern int UpperRSILevel      = 75;     // level for sells to take place
extern int LowerRSILevel      = 25;     // level for buys to take place
extern int ShortStochK        = 10;     // stochastic setting on "shortterm" time frame
extern int ShortStochD        = 3;      // stochastic setting on "shortterm" time frame
extern int ShortStochSlowing  = 2;      // stochastic setting on "shortterm" time frame
extern int LongStochK         = 60;     // stochastic setting on "longterm" time frame
extern int LongStochD         = 5;      // stochastic setting on "longterm" time frame
extern int LongStochSlowing   = 10;     // stochastic setting on "longterm" time frame
extern int UpperStochLevel    = 80;     // level for sells to take place
extern int LowerStochLevel    = 20;     // level for buys to take place
extern int buyterm            = 15;     // time frame that buys and sells take place on, 5 = 5M, 30 = 30M, etc.
extern int closeterm          = 240;     // time frame for longer term trend that allows for trading to occur 
extern int CloseBuyLevel      = 90;     // level where buys close when StochSignal begins to drop
extern int CloseSellLevel     = 10;     // level where sells close when tochSignal begins to rise
extern int startbuy           = 5;      // allows trading when longstoch gets below this level
extern int startsell          = 95;     // allows trading when longstoch gets above this level

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }

//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double  sendbuy, sendsell, leveler, lotcost, maxlots;
   double  i, previousrsi, currentrsi, previousshortstoch, currentshortstoch;
   double  previouslongstoch, currentlongstoch, lots, AB, AE;
   double  buytakeprofit, selltakeprofit, maxmargin;
   AE   = AccountEquity();
   AB   = AccountBalance();
            
   if (WantTakeProfit) 
     { 
      buytakeprofit  = Ask+Point*TakeProfit;
      selltakeprofit = Bid-Point*TakeProfit;
     }
        
//---- go trading only for first ticks of new bar
   if(Volume[0]>vol) return;
//---- RSI indicator
   currentrsi  = iRSI(NULL,buyterm,RSIperiod,PRICE_CLOSE,0);
   previousrsi = iRSI(NULL,buyterm,RSIperiod,PRICE_CLOSE,1); 
 //---- short Stochastic indicator
   currentshortstoch  = iStochastic(NULL,buyterm,ShortStochK,ShortStochD,ShortStochSlowing,MODE_LWMA,0,MODE_SIGNAL,0);
   previousshortstoch = iStochastic(NULL,buyterm,ShortStochK,ShortStochD,ShortStochSlowing,MODE_LWMA,0,MODE_SIGNAL,1);

 //---- long Stochastic indicator
   currentlongstoch  = iStochastic(NULL,closeterm,LongStochK,LongStochD,LongStochSlowing,MODE_LWMA,0,MODE_SIGNAL,0);
   previouslongstoch = iStochastic(NULL,closeterm,LongStochK,LongStochD,LongStochSlowing,MODE_LWMA,0,MODE_SIGNAL,1);
  
 //---- lot setting and modifications
   if (AccountLeverage() == 100) lotcost = 1000;
   if (AccountLeverage() == 200) lotcost = 50;
   maxmargin = NormalizeDouble(AB*(PercentRisk/100), 0);
   maxlots = NormalizeDouble(maxmargin/lotcost, 2);
   lots = NormalizeDouble(maxlots/LotDivider, 2);
      
   if(WantBuys && !WantSells) 
     {
     if(currentlongstoch < 30) lots = NormalizeDouble(lots*2, 2);
     if(currentlongstoch > 50 && currentlongstoch < 70) lots = NormalizeDouble(lots*0.5, 2);
     }
   if(!WantBuys && WantSells) 
     {
     if(currentlongstoch > 70) lots = NormalizeDouble(lots*2, 2);
     if(currentlongstoch > 30 && currentlongstoch < 50) lots = NormalizeDouble(lots*0.5, 2);
     }
   if (WantBuys && WantSells) lots = NormalizeDouble(lots/2, 2);
   if (lots < 0.01) lots = 0.01;
   if (lots > LargestLotAllowed) lots = LargestLotAllowed;
  
//---- buy conditions
  if(WantBuys)
   { 
    if(currentlongstoch < startbuy || previouslongstoch < currentlongstoch && currentlongstoch < 70)  
     {
     if(currentrsi < LowerRSILevel && currentshortstoch < LowerStochLevel)  
      {
      sendbuy=OrderSend(Symbol(),OP_BUY,lots,Ask,3,0,buytakeprofit,"",MAGICMA,0,SteelBlue);
      return;
      }
     }
   } 
//---- sell conditions 
  if(WantSells)
   { 
    if(currentlongstoch > startsell || previouslongstoch > currentlongstoch && currentlongstoch > 30)  
     {
     if(currentrsi > UpperRSILevel && currentshortstoch > UpperStochLevel)  
      {
      sendsell=OrderSend(Symbol(),OP_SELL,lots,Bid,3,0,selltakeprofit,"",MAGICMA,0,Orange);
      return;
      }
     }
   }     
//----
  }
  
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double previousstoch, currentstoch;
   int    sendclose, i, j, k, ticket, totalorders;
   string Name = "DropKickv16";
      
//---- Stochastic indicator
   currentstoch=iStochastic(NULL,closeterm,LongStochK,LongStochD,LongStochSlowing,MODE_LWMA,0,MODE_SIGNAL,0);
   previousstoch=iStochastic(NULL,closeterm,LongStochK,LongStochD,LongStochSlowing,MODE_LWMA,0,MODE_SIGNAL,1);
//---- close buy conditions 
    if(currentstoch > CloseBuyLevel && previousstoch > currentstoch)  
     {
     totalorders = OrdersTotal();
     for(i=totalorders-1;i>=0;i--)                    // scan all orders and positions...
       {
        OrderSelect(i, SELECT_BY_POS);
        if ( OrderSymbol()==Symbol() && ( (OrderMagicNumber() == MAGICMA) || (OrderComment() == Name)) )  
         { 
         if (OrderType() == OP_BUY)
          { 
          if (OrderProfit() > CloseAllAbove) OrderClose(OrderTicket(),OrderLots(),Bid,0,Yellow);
          }
         }
       }
      return;
     }
    if(currentstoch < CloseSellLevel && previousstoch < currentstoch)
     {
     totalorders = OrdersTotal();
     for(i=totalorders-1;i>=0;i--)                    // scan all orders and positions...
       {
        OrderSelect(i, SELECT_BY_POS);
        if ( OrderSymbol()==Symbol() && ( (OrderMagicNumber() == MAGICMA) || (OrderComment() == Name)) )  
         { 
          if (OrderType() == OP_SELL)
          { 
          if (OrderProfit() > CloseAllAbove) OrderClose(OrderTicket(),OrderLots(),Ask,0,Yellow);
          }
         }
       }
      return;
     }
  }

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100) return;
   
   double  i, OpenBuyLots, BuyOrders, OpenSellLots, SellOrders, leveler, MaxLotsAllowed;
   double  TotalProfit, BuyProfit, SellProfit, BuySwap, SellSwap, OpenLots, TotalLotsOpen;
   double  CurrentTime = (TimeHour(CurTime())*3600+TimeMinute(CurTime())*60);
   double  lotcost, maxlots, maxmargin, lots, AB, AE;
   
   AE   = AccountEquity();
   AB   = AccountBalance();
   if (AccountLeverage() == 100) lotcost = 1000;
   if (AccountLeverage() == 200) lotcost = 50;
   maxmargin = NormalizeDouble(AB*(PercentRisk/100), 0);
   maxlots = NormalizeDouble(maxmargin/lotcost, 2);
   lots = NormalizeDouble(maxlots/LotDivider, 2);
   
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      TotalLotsOpen += OrderLots();
       if (OrderSymbol() == Symbol())
        {
         if (OrderType() == OP_BUY)
          {
           OpenBuyLots += OrderLots();  
           BuyOrders ++;
           BuyProfit +=  OrderProfit();
           BuySwap += OrderSwap();
          }
         if (OrderType() == OP_SELL)
          {
           OpenSellLots += OrderLots();  
           SellOrders ++;
           SellProfit +=  OrderProfit();
           SellSwap += OrderSwap();
          }
        }
     }
   OpenLots = OpenBuyLots + OpenSellLots;
   BuyProfit = BuyProfit + BuySwap;
   SellProfit = SellProfit + SellSwap;  
   TotalProfit = BuyProfit + SellProfit + BuySwap + SellSwap;
    
   CheckForClose();
   
   if (AccountLeverage() == 200) leveler = 4;    // for mini accounts
   if (AccountLeverage() == 100) leveler = 0.1;  // for standard accounts
   if (AccountLeverage() == 50)  leveler = 0.2;  // for standard accounts
   //MaxLotsAllowed = ((AccountBalance()/AccountLeverage())*(PercentRisk/100))*leveler;
   
   if(lots > LargestLotAllowed) return;
   if(OpenLots > maxlots) return;
   if(lots + TotalLotsOpen > maxlots) return;
   if(TotalLotsOpen > maxlots) return;
   if(OpenBuyLots >= maxlots/2) return;
   if(OpenSellLots >= maxlots/2) return;
   if(AccountEquity() <= AccountBalance()/2) return;
   
   CheckForOpen();
                                      
 Comment(" BuyOrders ", BuyOrders,NL,
         " OpenBuyLots ", OpenBuyLots,NL,
         " BuyProfit ", BuyProfit,NL,
         " SellOrders ", SellOrders,NL,
         " OpenSellLots ", OpenSellLots,NL,
         " SellOrders ", SellOrders,NL,
         " TotalLotsOpen ", TotalLotsOpen,NL,
         " TotalProfit ", TotalProfit,NL,
         " Current Time is  ",TimeHour(CurTime()),":",TimeMinute(CurTime()),".",TimeSeconds(CurTime()));
  }
//+------------------------------------------------------------------+