//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
//|            Falcon S2 
//|   Last Updated 10-22-2006 10:30pm
//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
#property copyright "Skyview Investment Group"
#property link      "http://www.Skyview-IG.com"
#define     NL    "\n" 

// user input
extern double Lots         =  0.05;     // how many lots to trade at a time 
extern double TargetLevel  =  3;       // 1 = Low Risk 2 = Medium Risk, 3 = High Risk
extern double TradeLevel   =  8;
extern int    Spacing      =  4;       // min distance above last high or below last low
extern bool   LotIncrease  =  true;  
extern bool   Boost        =  true;    // buy more as the distance outside the band increases
extern bool   EndTrading   =  false;   // closes EA after all trades close
extern double EndHour      =  14;      // stop trading after this time friday

// non-external flag settings
int    EndDay       =    365;
int    Slippage     =    0;        // how many pips of slippage can you tolorate
bool   OneOrderOnly =    false ;   // one order at a time or not
int    LPeriod      =       1;     // Level period
int    Deviation    =       1;     // Level deviation
double BoostTrack;                 // used to track compounding and steps

// naming and numbering
int    MagicNumber  =   32526664642;        // allows multiple experts to trade on same account
string TradeComment = "_FALCON S2.txt";  // comment so multiple EAs can be seen in Account History
double StartingBalance = 0;                 // lot size control if LotIncrease == true

// Trade control
bool   TradeAllowed=true;                // used to manage trades

// Min/Max tracking and tick logging
int    maxOrders;                        // statistic for maximum numbers or orders open at one time
double maxEquity;                        // statistic for maximum equity level
double minEquity;                        // statistic for minimum equity level
double maxOEquity;                       // statistic for maximum equity level per order
double minOEquity;                       // statistic for minimum equity level per order 
double EquityPos=0;                      // statistic for number of ticks order was positive
double EquityNeg=0;                      // statistic for number of ticks order was negative
double EquityZer=0;                      // statistic for number of ticks order was zero

// used for verbose error logging
#include <stdlib.mqh>

//+-------------+
//| Custom init |
//|-------------+
// Called ONCE when EA is added to chart or recompiled

int init()
  {
   if(LotIncrease)
     {
      StartingBalance=AccountBalance()/Lots;
     }
  }

//+----------------+
//| Custom DE-init |
//+----------------+
// Called ONCE when EA is removed from chart

int deinit()
  {

  }
  
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+
void CloseAllThisSymbolBuy()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)  // only look if mygrid and symbol...
      {
           //Close opened long positions
           if ( OrderType() == OP_BUY )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 5, Red );
      }
  }
  return; 
  }
   
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+
void CloseAllThisSymbolSell()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)  // only look if mygrid and symbol...
      {
           //Close opened short positions
           if ( OrderType() == OP_SELL )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 5, Red );
      }
  }
  return; 
  }
   
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
  double  i, OpenBuyLots, BuyOrders, OpenSellLots, SellOrders;
  double  TotalProfit, BuyProfit, SellProfit, BuyTarget, SellTarget;
     
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
       if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
        {
         if (OrderType() == OP_BUY)  BuyProfit  = BuyProfit +  OrderProfit() + OrderSwap() + OrderCommission();
         if (OrderType() == OP_SELL) SellProfit = SellProfit + OrderProfit() + OrderSwap() + OrderCommission();
         if (OrderType() == OP_BUY)  BuyOrders++;
         if (OrderType() == OP_SELL) SellOrders++;
        }
     }
    BuyTarget  = Lots * TargetLevel*10 * BuyOrders;   
    SellTarget = Lots * TargetLevel*10 * SellOrders;

  //---- close buy conditions 
    if(BuyProfit  > BuyTarget)  CloseAllThisSymbolBuy(); 
    if(SellProfit > SellTarget) CloseAllThisSymbolSell();
    return;
  }
  
//+------------------------------------------------------------------+
//| Check for BUY order conditions                                   |
//+------------------------------------------------------------------+
void CheckForBuy ()
  {
   double   i, BuyPips, BuyLots;
   int      OrdersBUY;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   double SL=0;
   double TP=0;
      
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_BUY) OrdersBUY++;
        }
     } 
   
   // Lot increasement based on AccountBalance when expert is started
   // this will trade 1.0, then 1.1, then 1.2 etc as account balance grows
   // or 0.9 then 0.8 then 0.7 as account balance shrinks 
   if(LotIncrease)
     {
      BuyLots=NormalizeDouble(AccountBalance()/StartingBalance,2);
     } 
   
   if(OrdersBUY > 1) Lots = NormalizeDouble(BuyLots + (BuyLots * (OrdersBUY * 0.05)),2);
   if(Lots>100.0) Lots=1050.0;
   
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }
     
    //ENTRY LONG (buy, Ask)
   if( TradeAllowed )
     {
      while(true)
        {
         ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,SL,TP,TradeComment,MagicNumber,Blue);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            TradeAllowed=false;
            break;
           }
        }//while
     }//BUYme
   }
   
//+------------------------------------------------------------------+
//| Check for SELL order conditions                                   |
//+------------------------------------------------------------------+
void CheckForSell ()
  {
   double   i, SellPips, SellLots;
   int      OrdersSELL;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   double SL=0;
   double TP=0;
      
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_SELL) OrdersSELL++;
        }
     } 
   
   // Lot increasement based on AccountBalance when expert is started
   // this will trade 1.0, then 1.1, then 1.2 etc as account balance grows
   // or 0.9 then 0.8 then 0.7 as account balance shrinks 
   if(LotIncrease)
     {
      SellLots=NormalizeDouble(AccountBalance()/StartingBalance,2);
     } 
   
   if(OrdersSELL > 1) Lots = NormalizeDouble(SellLots + (SellLots * (OrdersSELL * 0.05)),2);
   if(Lots>100.0) Lots=100.0;
   
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }
     
   //ENTRY SHORT (sell, Bid)
   if( TradeAllowed )
     {
      while(true)
        {
         ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,SL,TP,TradeComment,MagicNumber,Red);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            TradeAllowed=false;
            break;
           }
        }//while
     }//SELLme
   }
    
//+-----------+
//| Main      |
//+-----------+
int start()
  {
  
   CheckForClose();
   
   // Is the EA still valid?
   if (DayOfYear() >= EndDay) 
      {
         Alert("EA has expired...");
         return;
      }
   
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   int      OrdersPerSymbol=0;
   int      OrdersBUY=0;
   int      OrdersSELL=0;
  
   // stoploss and takeprofit and close control
   double LowestBuy = 10;
   double HighestSell;
   double CurrentTime = (TimeHour(CurTime()));

   // direction control
   bool BUYme=false;
   bool SELLme=false;
      
   OrdersPerSymbol=0;
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_BUY)  OrdersBUY++;
         if(OrderType()==OP_SELL) OrdersSELL++;
        }
     }
    
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }

//+------------------------------------------------------------------+
//  Determine last order price                                       |
//-------------------------------------------------------------------+
      for(int i=0;i<OrdersTotal();i++)
      {
         OrderSelect(i,SELECT_BY_POS,MODE_TRADES);
         if(OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && OrderType()==OP_BUY && OrderOpenPrice()<LowestBuy) LowestBuy=OrderOpenPrice();
         if(OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber && OrderType()==OP_SELL && OrderOpenPrice()>HighestSell) HighestSell=OrderOpenPrice();
      }               
   Comment(" HighestSell ", HighestSell,NL,
         " LowestBuy ", LowestBuy);
         
   double bup=iBands(Symbol(),0,LPeriod,Deviation,0,PRICE_OPEN,MODE_UPPER,0);
   double bdn=iBands(Symbol(),0,LPeriod,Deviation,0,PRICE_OPEN,MODE_LOWER,0);
   
     if(Close[0]>bup+(TradeLevel*Point) && Ask > HighestSell + (Spacing+1)*Point) 
       {
       SELLme=true; 
       if(Boost && BoostTrack>0 && Close[0]>BoostTrack)
        {
         SELLme=true; 
         TradeAllowed=true;
        }
     }
     
    if(Close[0]<bdn-(TradeLevel*Point) && Ask < LowestBuy - (Spacing+1)*Point) 
      {
       BUYme=true;  
       if(Boost && BoostTrack>0 && Close[0]<BoostTrack)
         {
         BUYme=true;
         TradeAllowed=true;
         }
      }

     
   if (EndTrading == true && OrdersSELL == 0) SELLme = false;
   if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersSELL == 0) SELLme =false; 

   if (EndTrading == true && OrdersBUY == 0) BUYme = false;
   if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersBUY == 0) BUYme =false; 
  
   if (BUYme == true)  CheckForBuy();

   if (SELLme == true) CheckForSell();
    
	return(0);
    
 } // start()

 




