//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
//|   Falcon Standard 
//|   Last Updated 10-15-2006 10:30pm
//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
#property copyright "Skyview Investment Group"
#property link      "http://www.Skyview-IG.com"

// user input
extern double Lots         =  0.1;       // how many lots to trade at a time 
extern double TargetLevel  =  3;       // 1 = Low Risk 2 = Medium Risk, 3 = High Risk
extern double TradeLevel   =  12;   
extern bool   LotIncrease  =  true;  
extern bool   Boost        =  true;    // buy more as the distance outside the band increases
extern bool   EndTrading   =  false;   // closes EA after all trades close
extern double EndHour      =  14;      // stop trading after this time friday

// non-external flag settings
int    AcctNumber   =    1262383;
int    EndDay       =    293;
int    Slippage     =    2;        // how many pips of slippage can you tolorate
bool   OneOrderOnly =    false ;   // one order at a time or not
double ProfitMade   =    5000;     // how much money do you expect to make
double LossLimit    =    5000;     // how much loss can you tolorate
int    LPeriod      =       1;     // Level period
int    Deviation    =       1;     // Level deviation
double BoostTrack;              // used to track compounding and steps

// naming and numbering
int    MagicNumber  =   3252665;        // allows multiple experts to trade on same account
string TradeComment = "_FALCON_Standard.txt";     // comment so multiple EAs can be seen in Account History
double StartingBalance=0;                // lot size control if LotIncrease == true

// Trade control
bool   TradeAllowed=true;                // used to manage trades

// Min/Max tracking and tick logging
int    maxOrders;                        // statistic for maximum numbers or orders open at one time
double maxEquity;                        // statistic for maximum equity level
double minEquity;                        // statistic for minimum equity level
double maxOEquity;                       // statistic for maximum equity level per order
double minOEquity;                       // statistic for minimum equity level per order 
double EquityPos=0;                      // statistic for number of ticks order was positive
double EquityNeg=0;                      // statistic for number of ticks order was negative
double EquityZer=0;                      // statistic for number of ticks order was zero

// used for verbose error logging
#include <stdlib.mqh>


//+-------------+
//| Custom init |
//|-------------+
// Called ONCE when EA is added to chart or recompiled

int init()
  {
   if(LotIncrease)
     {
      StartingBalance=AccountBalance()/Lots;
     }
  }

//+----------------+
//| Custom DE-init |
//+----------------+
// Called ONCE when EA is removed from chart

int deinit()
  {

  }
  
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+

void CloseAllThisSymbolBuy()
{
  int totalorders = OrdersTotal();
  double tradesinprofit;
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)  // only look if mygrid and symbol...
      {
           //Close opened long positions
           if ( OrderType() == OP_BUY )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 5, Red );
           //Close opened short positions
           //if ( OrderType() == OP_SELL )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 5, Red );
      }
  }
  return; 
  }
   
   //+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+

void CloseAllThisSymbolSell()
{
  int totalorders = OrdersTotal();
  double tradesinprofit;
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)  // only look if mygrid and symbol...
      {
           //Close opened long positions
           //if ( OrderType() == OP_BUY )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 5, Red );
           //Close opened short positions
           if ( OrderType() == OP_SELL )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 5, Red );
      }
  }
  return; 
  }
   
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
  double  i, OpenBuyLots, BuyOrders, OpenSellLots, SellOrders;
  double  TotalProfit, BuyProfit, SellProfit, BuyTarget, SellTarget;
     
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
       if (OrderSymbol()==Symbol() && OrderMagicNumber() == MagicNumber)
        {
         if (OrderType() == OP_BUY)  BuyProfit  = BuyProfit +  OrderProfit() + OrderSwap() + OrderCommission();
         if (OrderType() == OP_SELL) SellProfit = SellProfit + OrderProfit() + OrderSwap() + OrderCommission();
         if (OrderType() == OP_BUY) BuyOrders++;
         if (OrderType() == OP_SELL) SellOrders++;
        }
     }
   BuyTarget  = Lots * TargetLevel;   
   SellTarget = Lots * TargetLevel;

  //---- close buy conditions 
    if(BuyProfit  > BuyTarget)  CloseAllThisSymbolBuy(); 
    if(SellProfit > SellTarget) CloseAllThisSymbolSell();
    return;
  }
  
//+------------------------------------------------------------------+
//| Check for BUY order conditions                                   |
//+------------------------------------------------------------------+
void CheckForBuy()
  {
   double   i, BuyPips;
   int      OrdersBUY;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   double   SL=0;
   double   TP=0;
   
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         if(OrderType()==OP_BUY) OrdersBUY++;
         OrdersPerSymbol++;
        }
     }
    if(OrdersBUY > 1)  Lots = Lots + (OrdersBUY * 0.1);

    if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }
       
  //ENTRY LONG (buy, Ask) 
   if( TradeAllowed )
     {
      while(true)
        {
         if(LossLimit ==0) SL=0; else SL=Ask-((LossLimit+10)*Point );
         if(ProfitMade==0) TP=0; else TP=Ask+((ProfitMade+10)*Point );
         ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,SL,TP,TradeComment,MagicNumber,White);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            TradeAllowed=false;
            break;
           }
            
        }//while   
     }//BUYme
   }
   
//+------------------------------------------------------------------+
//| Check for SELL order conditions                                   |
//+------------------------------------------------------------------+
void CheckForSell ()
  {
   double   i, SellPips;
   int      OrdersSELL;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   double SL=0;
   double TP=0;
      
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_SELL) {OrdersSELL++;}
        }
     } 
    
    if(OrdersSELL > 1) Lots = Lots + (OrdersSELL * 0.1);

   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }
     
    //ENTRY SHORT (sell, Bid)
   if( TradeAllowed )
     {
      while(true)
        {
         if(LossLimit ==0) SL=0; else SL=Bid+((LossLimit+10)*Point );
         if(ProfitMade==0) TP=0; else TP=Bid-((ProfitMade+10)*Point );
         ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,SL,TP,TradeComment,MagicNumber,Red);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            TradeAllowed=false;
            break;
           }
        }//while
     }//SELLme
   }
    
//+-----------+
//| Main      |
//+-----------+
// Called EACH TICK and each Bar[]

int start()
  {
   CheckForClose();
   
   // Is the EA still valid?
   if (DayOfYear() >= EndDay) 
      {
         Alert("EA has expired...");
         return;
      }

   if (AcctNumber != AccountNumber())
      {
         Alert("Account not authorized...");
         return;
      }
    Comment(DayOfYear());
   
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   int      OrdersPerSymbol=0;
   int      OrdersBUY=0;
   int      OrdersSELL=0;
  
   // stoploss and takeprofit and close control
   double SL=0;
   double TP=0;
   double CurrentProfit=0;
   double CurrentBasket=0;
   double CurrentTime = (TimeHour(CurTime()));

   // direction control
   bool BUYme=false;
   bool SELLme=false;
      
   // Lot increasement based on AccountBalance when expert is started
   // this will trade 1.0, then 1.1, then 1.2 etc as account balance grows
   // or 0.9 then 0.8 then 0.7 as account balance shrinks 
   if(LotIncrease)
     {
      Lots=NormalizeDouble(AccountBalance()/StartingBalance,2);
      if(Lots>100.0 ) Lots=100.0;
     }

   OrdersPerSymbol=0;
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_BUY) {OrdersBUY++;}
         if(OrderType()==OP_SELL){OrdersSELL++;}
        }
     }
    
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      TradeAllowed=true;
     }
     
   double bup=iBands(Symbol(),0,LPeriod,Deviation,0,PRICE_OPEN,MODE_UPPER,0);
   double bdn=iBands(Symbol(),0,LPeriod,Deviation,0,PRICE_OPEN,MODE_LOWER,0);
   
   if(Close[0]>bup+(TradeLevel*Point)) 
     {
      SELLme=true; 
      if(Boost && BoostTrack>0 && Close[0]>BoostTrack)
        {
         SELLme=true; 
         TradeAllowed=true;
        }
     }
     
   if(Close[0]<bdn-(TradeLevel*Point)) 
     {
      BUYme=true;  
      if(Boost && BoostTrack>0 && Close[0]<BoostTrack)
        {
         BUYme=true;
         TradeAllowed=true;
        }
     }

   if (EndTrading == true && OrdersSELL == 0) SELLme = false;
   if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersSELL == 0) SELLme =false; 

   if (EndTrading == true && OrdersBUY == 0) BUYme = false;
   if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersBUY == 0) BUYme =false; 

   if (SELLme == true) CheckForSell();
   if (BUYme == true)  CheckForBuy();

 } // start()

 




