//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
//|   Raptor 
//|   Last Updated 10-15-2006 10:30pm
//|$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
#property copyright "Skyview Investment Group"
#property link      "http://www.Skyview-IG.com"

// external variables
extern double Lots         =  0.1;     // number of lots to trade  
extern bool   LotIncrease  =  true ;   // increase lots based on balance 
extern bool   Boost        =  true;    // buy more as the distance increases
extern bool   EndTrading   =  false;   // closes EA after all trades close
extern double EndHour      =  14;      // stop trading after this time friday


// internal varibles
int    AcctNumber   =    1256437;
int    EndDay       =    294;
bool   logging      =    false ;   // log data or not
bool   logerrs      =    false ;   // log errors or not
bool   logtick      =    false ;   // log tick data while orders open (or not)
int    Deviation    =    3;        // Bollinger deviation
int    BPeriod      =    2;        // Bollinger period
double BDistance    =    7;        // plus how much
double ProfitMade   =    0.1;      // how much money do you expect to make
double LossLimit    =    0;       // how much loss can you take
int    Slippage     =    2;        // how many pips of slippage can you tolorate
double BoostTrack;                 // used to track compounding and steps

int    MagicNumber      =  727867;  // allows multiple experts to trade on same account
string TradeComment     = "Raptor";
double StartingLots     = 0;         // lot size control if LotIncrease == true

bool   TradeAllowed     = true;     


// Min/Max tracking and tick logging
int    maxOrders;                        // statistic for maximum numbers or orders open at one time
double maxEquity;                        // statistic for maximum equity level
double minEquity;                        // statistic for minimum equity level
double maxOEquity;                       // statistic for maximum equity level per order
double minOEquity;                       // statistic for minimum equity level per order 
double EquityPos=0;                      // statistic for number of ticks order was positive
double EquityNeg=0;                      // statistic for number of ticks order was negative
double EquityZer=0;                      // statistic for number of ticks order was zero

// used for verbose error logging
#include <stdlib.mqh>

//+-------------+
//| Custom init |
//|-------------+
// Called ONCE when EA is added to chart or recompiled

int init()
  {
   if(LotIncrease)
     {
      StartingLots=AccountBalance()/Lots;
     }
  }

//+----------------+
//| Custom DE-init |
//+----------------+
// Called ONCE when EA is removed from chart

int deinit()
  {

   // always indicate deinit statistics
   logwrite(TradeComment,"MAX number of orders "+maxOrders);
   logwrite(TradeComment,"MAX equity           "+maxEquity);
   logwrite(TradeComment,"MIN equity           "+minEquity);

   // so you can see stats in journal
   Print("MAX number of orders "+maxOrders);
   Print("MAX equity           "+maxEquity);
   Print("MIN equity           "+minEquity);


   logwrite(TradeComment,"DE-Init Complete");
   
   Comment(" ");
  }

//+------------------------------------------------------------------+
//| Check for BUY order conditions                                   |
//+------------------------------------------------------------------+
void CheckForBuy()
  {
   double   i, BuyPips;
   int      OrdersBUY;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
    // stoploss and takeprofit and close control
   double SL=0;
   double TP=0;
   
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
        }
     }
    
    if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      
      // This allows the 1st order at BDistance after everything closes
      // others are handled in the indicator section 
      TradeAllowed=true;
     }
       
  //ENTRY LONG (buy, Ask) 
   if( TradeAllowed )
     {
      while(true)
        {
         if(LossLimit ==0) SL=0; else SL=Ask-((LossLimit+10)*Point );
         if(ProfitMade==0) TP=0; else TP=Ask+((ProfitMade+10)*Point );
         ticket=OrderSend(Symbol(),OP_BUY,Lots,Ask,Slippage,SL,TP,TradeComment,MagicNumber,White);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            //CompoundTrack=Ask;
            maxOEquity=0;
            minOEquity=0;
            EquityPos=0;
            EquityNeg=0;
            EquityZer=0;
            TradeAllowed=false;
            break;
           }
            
        }//while   
     }//BUYme
   }
   
//+------------------------------------------------------------------+
//| Check for SELL order conditions                                   |
//+------------------------------------------------------------------+
void CheckForSell ()
  {
   double   i, SellPips;
   int      OrdersSELL;
   int      OrdersPerSymbol;
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
     // stoploss and takeprofit and close control
   double SL=0;
   double TP=0;
      
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_SELL) {OrdersSELL++;}
        }
     } 
    
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      
      // This allows the 1st order at BDistance after everything closes
      // others are handled in the indicator section 
      TradeAllowed=true;
     }
     
    //ENTRY SHORT (sell, Bid)
   if( TradeAllowed )
     {
      while(true)
        {
         if(LossLimit ==0) SL=0; else SL=Bid+((LossLimit+10)*Point );
         if(ProfitMade==0) TP=0; else TP=Bid-((ProfitMade+10)*Point );
         ticket=OrderSend(Symbol(),OP_SELL,Lots,Bid,Slippage,SL,TP,TradeComment,MagicNumber,Red);
         gle=GetLastError();
         if(gle==0)
           {
            BoostTrack=Close[0];
            //CompoundTrack=Bid;
            maxOEquity=0;
            minOEquity=0;
            EquityPos=0;
            EquityNeg=0;
            EquityZer=0;
            TradeAllowed=false;
            break;
           }
            
        }//while
     }//SELLme
   }
      
//+-----------+
//| Main      |
//+-----------+
// Called EACH TICK and each Bar[]

int start()
  {
    
   // Is the EA still valid?
   
   if (DayOfYear() >= EndDay) 
      {
         Alert("EA has expired...");
         return;
      }

   if (AcctNumber != AccountNumber())
   
      {
         Alert("Account not authorized...");
         return;
      }
   
   Comment(DayOfYear());
   
   int      cnt=0;
   int      gle=0;
   int      ticket=0;
   int      OrdersPerSymbol=0;
   int      OrdersBUY=0;
   int      OrdersSELL=0;
   double   CurrentTime = (TimeHour(CurTime()));

   double CurrentProfit=0;
   double CurrentBasket=0;
   
   // direction control
   bool BUYme=false;
   bool SELLme=false;
    // stoploss and takeprofit and close control
   double SL=0;
   double TP=0;

   // Lot increasement based on AccountBalance when expert is started
   // this will trade 1.0, then 1.1, then 1.2 etc as account balance grows
   // or 0.9 then 0.8 then 0.7 as account balance shrinks 
   if(LotIncrease)
     {
      Lots=NormalizeDouble(AccountBalance()/StartingLots,2);
      if(Lots>100.0) Lots=100.0;
     }

   OrdersPerSymbol=0;
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
        {
         OrdersPerSymbol++;
         if(OrderType()==OP_BUY) {OrdersBUY++;}
         if(OrderType()==OP_SELL){OrdersSELL++;}
        }
     }
   
   if(OrdersPerSymbol==0)
     {
      BoostTrack=0;
      
      // This allows the 1st order at BDistance after everything closes
      // others are handled in the indicator section 
      TradeAllowed=true;
     }

   // keep some statistics
   if(OrdersPerSymbol>maxOrders) maxOrders=OrdersPerSymbol;

   //+-----------------------------+
   //| Insert your indicator here  |
   //| And set either BUYme or     |
   //| SELLme true to place orders |
   //+-----------------------------+
   
   double bup=iBands(Symbol(),0,BPeriod,Deviation,0,PRICE_OPEN,MODE_UPPER,0);
   double bdn=iBands(Symbol(),0,BPeriod,Deviation,0,PRICE_OPEN,MODE_LOWER,0);

   if(Close[0]>bup)                 Comment("S Up="+bup+" Dn="+bdn);
   if(Close[0]<bdn)                 Comment("B Up="+bup+" Dn="+bdn);
   if(Close[0]<bup && Close[0]>bdn) Comment("- Up="+bup+" Dn="+bdn);
   
   // if close is above upper band + BDistance then SELL
   if(Close[0]>bup+(BDistance*Point)) 
     {
      SELLme=true; 
      logwrite("BollTick3g.txt","---SELLme happened"); 
      if(Boost && BoostTrack>0 && Close[0]>BoostTrack)
        {
         SELLme=true; 
         TradeAllowed=true;
         logwrite("BollTick3h.txt","---SELLme happened again "+BoostTrack ); 
        }
     }

   // if close is below lower band + BDistance then BUY
   if(Close[0]<bdn-(BDistance*Point)) 
     {
      BUYme=true;  
      logwrite("BollTick3h.txt","----BUYme happened"); 
      if(Boost && BoostTrack>0 && Close[0]<BoostTrack)
        {
         BUYme=true;
         TradeAllowed=true;
         logwrite("BollTick3g.txt","----BUYme happened again "+BoostTrack ); 
        }
     }
    if (EndTrading == true && OrdersSELL == 0) SELLme = false;
    if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersSELL == 0) SELLme =false; 

    if (EndTrading == true && OrdersBUY == 0) BUYme = false;
    if (DayOfWeek()>=5 && CurrentTime >= EndHour && OrdersBUY == 0) BUYme =false; 

    if(TradeAllowed && BUYme)  CheckForBuy();
    if(TradeAllowed && SELLme) CheckForSell();
    
   //+------------+
   //| End Insert |
   //+------------+

       
   // accumulate statistics
   CurrentBasket=AccountEquity()-AccountBalance();
   if(CurrentBasket>maxEquity) { maxEquity=CurrentBasket; maxOEquity=CurrentBasket; }
   if(CurrentBasket<minEquity) { minEquity=CurrentBasket; minOEquity=CurrentBasket; }
   if(CurrentBasket>0)  EquityPos++;
   if(CurrentBasket<0)  EquityNeg++;
   if(CurrentBasket==0) EquityZer++;
   
   //
   // Order Management
   //
   for(cnt=OrdersTotal();cnt>=0;cnt--)
     {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if( OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber )
        {
        
         if(OrderType()==OP_BUY)
           {
            CurrentProfit=(Bid-OrderOpenPrice()) ;
            if(logtick) logwrite(TradeComment,"BUY  CurrentProfit="+CurrentProfit/Point+" CurrentBasket="+CurrentBasket/Point);

            //
            // Did we make a profit
            //======================
            //
            if(ProfitMade>0 && CurrentProfit>=(ProfitMade*Point))
              {
               while(true)
                 {
                  OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,White);
                  gle=GetLastError();
                  if(gle==0)
                    {
                     if(logging) logwrite(TradeComment,"CLOSE BUY PROFIT Ticket="+OrderTicket()+" SL="+SL+" TP="+TP);
                     break;
                    }
                   
                 }//while
              }//if
              

            //
            // Did we take a loss
            //====================
            //
            if( LossLimit>0 && CurrentProfit<=(LossLimit*(-1)*Point)  )
              {
               while(true)
                 {
                  OrderClose(OrderTicket(),OrderLots(),Bid,Slippage,White);
                  gle=GetLastError();
                  if(gle==0)
                    {
                     if(logging) logwrite(TradeComment,"CLOSE BUY LOSS Ticket="+OrderTicket()+" SL="+SL+" TP="+TP);
                     break;
                    }
                   
                 }//while
                 
              }//if
              
           } // if BUY


         if(OrderType()==OP_SELL)
           {
            CurrentProfit=(OrderOpenPrice()-Ask);
            if(logtick) logwrite(TradeComment,"SELL CurrentProfit="+CurrentProfit/Point+" CurrentBasket="+CurrentBasket/Point);

            //
            // Did we make a profit
            //======================
            //
            if( ProfitMade>0 && CurrentProfit>=(ProfitMade*Point) )
              {
               while(true)
                 {
                  OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Red);
                  gle=GetLastError();
                  if(gle==0)
                    {
                     if(logging) logwrite(TradeComment,"CLOSE SELL PROFIT Ticket="+OrderTicket()+" SL="+SL+" TP="+TP);
                    break;
                    }
                    
                 }//while                 
              }//if


            //
            // Did we take a loss
            //====================
            //
            if( LossLimit>0 && CurrentProfit<=(LossLimit*(-1)*Point) )
              {
               while(true)
                 {
                  OrderClose(OrderTicket(),OrderLots(),Ask,Slippage,Red);
                  gle=GetLastError();
                  if(gle==0)
                    {
                     if(logging) logwrite(TradeComment,"CLOSE SELL LOSS Ticket="+OrderTicket()+" SL="+SL+" TP="+TP);
                    break;
                    }
                     
                 }//while

              }//if

           } //if SELL
           
        } // if(OrderSymbol)
        
     } // for

  } // start()


void logwrite (string filename, string mydata)
  {
   int myhandle;
   string gregorian=TimeToStr(CurTime(),TIME_DATE|TIME_SECONDS);
   
   myhandle=FileOpen(Symbol()+"_"+filename, FILE_CSV|FILE_WRITE|FILE_READ, ";");
   if(myhandle>0)
     {
      FileSeek(myhandle,0,SEEK_END);
      FileWrite(myhandle, mydata+" "+gregorian);
      FileClose(myhandle);
     }
  } 


