//+------------------------------------------------------------------+
//|                                               SimpleStochv3c.mq4 |
//|                                       Copyright  2006, kmrunner |
//|                                                   August 9, 2006 |
//+------------------------------------------------------------------+
#define MAGICMA  20060809
#include <stdlib.mqh>
#define     NL    "\n"

extern double ratio             = 1.01;   // Equity/Balance ratio or profit level to take profit
extern double LotDivider        = 20; 
extern double PercentRisk       = 20; 
extern bool SplitTheRisk        = false;
extern int vol                  = 8;      // sets trades for this many ticks of current bar
extern int timeframe            = 60;     // time frame for longer term trend that allows for trading to occur 
extern int stopbuy              = 20;     // allows trading when longstoch gets below this level
extern int stopsell             = 80;     // allows trading when longstoch gets above this level
extern int StochK               = 40;     // stochastic setting on "longterm" time frame
extern int StochD               = 5;     // stochastic setting on "longterm" time frame
extern int StochSlowing         = 8;      // stochastic setting on "longterm" time frame
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double  previousstoch, currentstoch, AB, maxmargin, AE, maxlots, lotcost, lots;
 //---- go trading only for first ticks of new bar
   if(Volume[0]>vol) return;
   
 //---- long Stochastic indicator
   currentstoch  = iStochastic(NULL,timeframe,StochK,StochD,StochSlowing,MODE_LWMA,0,MODE_SIGNAL,1);
   previousstoch = iStochastic(NULL,timeframe,StochK,StochD,StochSlowing,MODE_LWMA,0,MODE_SIGNAL,2);
 //---- lot setting and modifications
   AE   = AccountEquity();
   AB   = AccountBalance();
   if (AccountLeverage() == 100) lotcost = 1000;
   if (AccountLeverage() == 200) lotcost = 50;
   maxmargin = NormalizeDouble(AB*(PercentRisk/100), 0);
   maxlots = NormalizeDouble(maxmargin/lotcost, 2);
   lots = NormalizeDouble(maxlots/LotDivider, 2);
  
 //---- buy conditions
   if(previousstoch<currentstoch && currentstoch < stopbuy)  
      {
      OrderSend(Symbol(),OP_BUY,lots,Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
      }
    
 //---- sell conditions 
    if(previousstoch>currentstoch && currentstoch > stopsell)  
      {
      OrderSend(Symbol(),OP_SELL,lots,Bid,3,0,0,"",MAGICMA,0,Orange);
      return;
      }
 //----
  }
  
//+------------------------------------------------------------------------+
//| cancels all pending orders and closes open positions                   |
//+------------------------------------------------------------------------+

void CloseAllThisSymbol()
{
  int totalorders = OrdersTotal();
  double tradesinprofit;
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
// modified by cori. Using OrderMagicNumber to identify the trades of the grid // hdb added or gridname for compatibility
    if ( OrderSymbol()==Symbol())  // only look if mygrid and symbol...
      {
           //Close opened long positions
           if ( OrderType() == OP_BUY )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_BID), 5, Red );
           //Close opened short positions
           if ( OrderType() == OP_SELL )  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(), MODE_ASK), 5, Red );
      }
  }
  return; 
  } 
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
  double  i, OpenBuyLots, BuyOrders, OpenSellLots, SellOrders;
   double  TotalProfit, BuyProfit, SellProfit;
     
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
       if (OrderSymbol() == Symbol())
        {
         if (OrderType() == OP_BUY)  BuyProfit  +=  OrderProfit();
         if (OrderType() == OP_SELL) SellProfit +=  OrderProfit();
        }
     }
   TotalProfit = BuyProfit + SellProfit;
    
  //---- close buy conditions 
    if(TotalProfit/AccountBalance() >= ratio) CloseAllThisSymbol(); 
    
    return;
  }

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100) return;
   
   double  i, OpenBuyLots, BuyOrders, OpenSellLots, SellOrders, leveler, MaxLotsAllowed;
   double  TotalProfit, BuyProfit, SellProfit, OpenLots, TotalLotsOpen;
   double  CurrentTime = (TimeHour(CurTime())*3600+TimeMinute(CurTime())*60);
   double  lotcost, maxlots, maxmargin, lots, AB, AE;
     
   AE   = AccountEquity();
   AB   = AccountBalance();
   if (AccountLeverage() == 100) lotcost = 1000;
   if (AccountLeverage() == 200) lotcost = 50;
   maxmargin = NormalizeDouble(AB*(PercentRisk/100), 0);
   maxlots = NormalizeDouble(maxmargin/lotcost, 2);
   lots = NormalizeDouble(maxlots/LotDivider, 2);
   
   for(i = 0; i < OrdersTotal(); i++)
     {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      TotalLotsOpen += OrderLots();
       if (OrderSymbol() == Symbol())
        {
         if (OrderType() == OP_BUY)
          {
           OpenBuyLots += OrderLots();  
           BuyOrders ++;
           BuyProfit +=  OrderProfit();
          }
         if (OrderType() == OP_SELL)
          {
           OpenSellLots += OrderLots();  
           SellOrders ++;
           SellProfit +=  OrderProfit();
          }
        }
     }
   OpenLots = OpenBuyLots + OpenSellLots;
   TotalProfit = BuyProfit + SellProfit;
    
   if (AccountLeverage() == 200) leveler = 4;    // for mini accounts
   if (AccountLeverage() == 100) leveler = 0.1;  // for standard accounts
   if (AccountLeverage() == 50)  leveler = 0.2;  // for standard accounts
   if(SplitTheRisk) 
      { 
      OpenLots = ((AccountBalance()/AccountLeverage())*(PercentRisk/100))*leveler;
      if(lots + TotalLotsOpen > OpenLots) return;
      if(TotalLotsOpen > OpenLots) return;
      }
     
   if (!SplitTheRisk) 
      { 
      MaxLotsAllowed = ((AccountBalance()/AccountLeverage())*(PercentRisk/100))*leveler;
      if(OpenLots > MaxLotsAllowed) return;
      if(lots + TotalLotsOpen > MaxLotsAllowed) return;
      if(TotalLotsOpen > MaxLotsAllowed) return;
      }
      
   CheckForClose();
     
   CheckForOpen();
                                      
 Comment(" BuyOrders ", BuyOrders,NL,
         " OpenBuyLots ", OpenBuyLots,NL,
         " BuyProfit ", BuyProfit,NL,
         " SellOrders ", SellOrders,NL,
         " OpenSellLots ", OpenSellLots,NL,
         " SellOrders ", SellOrders,NL,
         " TotalLotsOpen ", TotalLotsOpen,NL,
         " TotalProfit ", TotalProfit,NL,
         " Current Time is  ",TimeHour(CurTime()),":",TimeMinute(CurTime()),".",TimeSeconds(CurTime()));
  }
//+------------------------------------------------------------------+